@@ -104,7 +104,7 @@ ggiNEXT.iNEXT <- function(x, type=1, se=TRUE, facet.var="none", color.var="site"
     z$col <- z$shape <- paste(z$site, z$order, sep="-")
   }
   
-  z$lty <- factor(z$method, c("interpolated", "extrapolated"), c("interpolation", "extrapolation"))
+  z$lty <- factor(z$method, c("interpolated", "observed", "extrapolated"), c("interpolation", "interpolation", "extrapolation"))
   z$col <- factor(z$col)
   data.sub <- z[which(z$method=="observed"),]
   
@@ -184,6 +184,7 @@ ggiNEXT.iNEXT <- function(x, type=1, se=TRUE, facet.var="none", color.var="site"
       theme(legend.position="bottom",
             legend.title=element_blank())
   }
+  g <- g + theme(legend.box = "vertical")
   return(g)
   
 }
