@@ -1,5 +1,5 @@
-invChat.Ind <- function(x, C)
-{
+invChat.Ind <- function(x, C, conf=NULL){
+  m <- NULL # no visible binding for global variable 'm'
   n <- sum(x)
   refC <- Chat.Ind(x,n)
   f <- function(m, C) abs(Chat.Ind(x,m)-C)
@@ -23,18 +23,35 @@ invChat.Ind <- function(x, C)
   }
   if(mm > 2*n) 
 	warning("The maximum size of the extrapolation exceeds double reference sample size, the results for q = 0 may be subject to large prediction bias.")
-  method <- ifelse(mm<n, "rarefaction", ifelse(mm==n, "observation", "extrapolation"))
-  out <- data.frame(m=mm, method=method, 
-                    "SC"=round(Chat.Ind(x,mm),4), 
-                    "q = 0"=round(Dqhat.Ind(x,0,mm),3),
-                    "q = 1"=round(Dqhat.Ind(x,1,mm),3),
-                    "q = 2"=round(Dqhat.Ind(x,2,mm),3))
-  colnames(out) <- c("m", "method", "SC", "q = 0", "q = 1", "q = 2")
+  method <- ifelse(mm<n, "interpolated", ifelse(mm==n, "observed", "extrapolated"))
+  # out <- data.frame(m=mm, method=method, 
+  #                   "SC"=round(Chat.Ind(x,mm),4), 
+  #                   "q = 0"=round(Dqhat.Ind(x,0,mm),3),
+  #                   "q = 1"=round(Dqhat.Ind(x,1,mm),3),
+  #                   "q = 2"=round(Dqhat.Ind(x,2,mm),3))
+  # colnames(out) <- c("m", "method", "SC", "q = 0", "q = 1", "q = 2")
+  # out
+  
+  if(is.null(conf)){
+    out <- data.frame(m=mm, method=method, 
+                      SamCov=round(Chat.Ind(x,mm),3),
+                      SpeRic=round(Dqhat.Ind(x,0,mm),3),
+                      ShaDiv=round(Dqhat.Ind(x,1,mm),3),
+                      SimDiv=round(Dqhat.Ind(x,2,mm),3))
+    colnames(out) <- c("m", "method", "SC", "q = 0", "q = 1", "q = 2")
+  }else{
+    tmp0 <- iNEXT.Ind(x, q=0, m=c(1,mm), se = TRUE, conf = conf)
+    tmp1 <- iNEXT.Ind(x, q=1, m=c(1,mm), se = TRUE, conf = conf)
+    tmp2 <- iNEXT.Ind(x, q=2, m=c(1,mm), se = TRUE, conf = conf)
+    tmp <- subset(rbind(tmp0,tmp1,tmp2), m==mm)
+    out <- tmp[,c(1,2,3,7,4,5,6)]
+    out[,4:7] <- round(out[,4:7],3)
+  }
   out
 }
 
-invChat.Sam <- function(x, C)
-{
+invChat.Sam <- function(x, C, conf=NULL){
+  m <- NULL # no visible binding for global variable 'm'
   n <- max(x)
   refC <- Chat.Sam(x,n)
   f <- function(m, C) abs(Chat.Sam(x,m)-C)
@@ -59,19 +76,78 @@ invChat.Sam <- function(x, C)
   }
   if(mm > 2*n) 
 	warning("The maximum size of the extrapolation exceeds double reference sample size, the results for q = 0 may be subject to large prediction bias.")
+
   
-  method <- ifelse(mm<n, "rarefaction", ifelse(mm==n, "observation", "extrapolation"))
-  out <- data.frame(t=mm, method=method, 
-                    "SC"=round(Chat.Sam(x,mm),4), 
-                    "q = 0"=round(Dqhat.Sam(x,0,mm),3),
-                    "q = 1"=round(Dqhat.Sam(x,1,mm),3),
-                    "q = 2"=round(Dqhat.Sam(x,2,mm),3))
-  colnames(out) <- c("t", "method", "SC", "q = 0", "q = 1", "q = 2")
+  if(is.null(conf)){
+    method <- ifelse(mm<n, "interpolated", ifelse(mm==n, "observed", "extrapolated"))
+    out <- data.frame(t=mm, method=method, 
+                      SamCov=round(Chat.Sam(x,mm),3),
+                      SpeRic=round(Dqhat.Sam(x,0,mm),3),
+                      ShaDiv=round(Dqhat.Sam(x,1,mm),3),
+                      SimDiv=round(Dqhat.Sam(x,2,mm),3))
+    colnames(out) <- c("t", "method", "SC", "q = 0", "q = 1", "q = 2")
+  }else{
+    tmp0 <- iNEXT.Sam(x, q=0, t=c(1,mm), se = TRUE, conf = conf)
+    tmp1 <- iNEXT.Sam(x, q=1, t=c(1,mm), se = TRUE, conf = conf)
+    tmp2 <- iNEXT.Sam(x, q=2, t=c(1,mm), se = TRUE, conf = conf)
+    tmp <- subset(rbind(tmp0,tmp1,tmp2), t==mm)
+    out <- tmp[,c(1,2,3,7,4,5,6)]
+    out[,4:7] <- round(out[,4:7],3)
+  }
   out
 }
 
 
+invSize.Ind <- function(x, size, conf=NULL){
+  m <- NULL # no visible binding for global variable 'm'
+  
+  if(is.null(size)){
+    size <- sum(x)
+  }
+  method <- ifelse(size<sum(x), "interpolated", ifelse(size==sum(x), "observed", "extrapolated"))
+  if(is.null(conf)){
+    out <- data.frame(m=size, method=method, 
+                      SamCov=round(Chat.Ind(x,size),3),
+                      SpeRic=round(Dqhat.Ind(x,0,size),3),
+                      ShaDiv=round(Dqhat.Ind(x,1,size),3),
+                      SimDiv=round(Dqhat.Ind(x,2,size),3))
+    colnames(out) <- c("m", "method", "SC", "q = 0", "q = 1", "q = 2")
+  }else{
+    tmp0 <- iNEXT.Ind(x, q=0, m=c(1,size), se = TRUE, conf = conf)
+    tmp1 <- iNEXT.Ind(x, q=1, m=c(1,size), se = TRUE, conf = conf)
+    tmp2 <- iNEXT.Ind(x, q=2, m=c(1,size), se = TRUE, conf = conf)
+    tmp <- subset(rbind(tmp0,tmp1,tmp2), m==size)
+    out <- tmp[,c(1,2,3,7,4,5,6)]
+    out[,4:7] <- round(out[,4:7],3)
+  }
+  out
+}
 
+invSize.Sam <- function(x, size, conf=NULL){
+  m <- NULL # no visible binding for global variable 'm'
+  
+  if(is.null(size)){
+    size <- max(x)
+  }
+  if(is.null(conf)){
+    method <- ifelse(size<max(x), "interpolated", ifelse(size==max(x), "observed", "extrapolated"))
+    out <- data.frame(t=size, method=method, 
+                      SamCov=round(Chat.Sam(x,size),3),
+                      SpeRic=round(Dqhat.Sam(x,0,size),3),
+                      ShaDiv=round(Dqhat.Sam(x,1,size),3),
+                      SimDiv=round(Dqhat.Sam(x,2,size),3))
+    colnames(out) <- c("m", "method", "SC", "q = 0", "q = 1", "q = 2")
+    out
+  }else{
+    tmp0 <- iNEXT.Sam(x, q=0, t=c(1,size), se = TRUE, conf = conf)
+    tmp1 <- iNEXT.Sam(x, q=1, t=c(1,size), se = TRUE, conf = conf)
+    tmp2 <- iNEXT.Sam(x, q=2, t=c(1,size), se = TRUE, conf = conf)
+    tmp <- subset(rbind(tmp0,tmp1,tmp2), t==size)
+    out <- tmp[,c(1,2,3,7,4,5,6)]
+    out[,4:7] <- round(out[,4:7],3)
+  }
+  out
+}
 #
 #
 ###############################################
@@ -89,7 +165,7 @@ invChat.Sam <- function(x, C)
 # 
 # @export
 
-invChat <- function(x, datatype="abundance", C=NULL){
+invChat <- function(x, datatype="abundance", C=NULL, conf=NULL){
   TYPE <- c("abundance", "incidence")
   if(is.na(pmatch(datatype, TYPE)))
     stop("invalid datatype")
@@ -102,42 +178,43 @@ invChat <- function(x, datatype="abundance", C=NULL){
       if(is.null(C)){
         C <- Chat.Ind(x, 2*sum(x))
       }
-      invChat.Ind(x, C)
+      out <- invChat.Ind(x, C, conf)
     }
     else if(class(x)=="list"){
       if(is.null(C)){
         C <- min(unlist(lapply(x, function(x) Chat.Ind(x, sum(x)))))
       }
-      do.call(rbind, lapply(x, function(x) invChat.Ind(x, C)))
+      out <- do.call(rbind, lapply(x, function(x) invChat.Ind(x, C, conf)))
     }else if(class(x)=="data.frame" | class(x)=="matrix"){
       if(is.null(C)){
         C <- min(unlist(apply(x, 2, function(x) Chat.Ind(x, sum(x)))))
       }
-      do.call(rbind, apply(x, 2, function(x) invChat.Ind(x, C)))
+      out <- do.call(rbind, apply(x, 2, function(x) invChat.Ind(x, C, conf)))
     }
   }else if(datatype=="incidence"){
     if(class(x)=="numeric" | class(x)=="integer"){
       if(is.null(C)){
         C <- Chat.Sam(x, 2*max(x))
       }
-      invChat.Sam(x, C)
+      out <- invChat.Sam(x, C, conf)
     }
     else if(class(x)=="list"){
       if(is.null(C)){
         C <- min(unlist(lapply(x, function(x) Chat.Sam(x, max(x)))))
       }
-      do.call(rbind, lapply(x, function(x) invChat.Sam(x, C)))
+      out <- do.call(rbind, lapply(x, function(x) invChat.Sam(x, C, conf)))
     }else if(class(x)=="data.frame" | class(x)=="matrix"){
       if(is.null(C)){
         C <- min(unlist(apply(x, 2, function(x) Chat.Sam(x, max(x)))))
       }
-      do.call(rbind, apply(x, 2, function(x) invChat.Sam(x, C)))
+      out <- do.call(rbind, apply(x, 2, function(x) invChat.Sam(x, C, conf)))
     }
-    
   }
+  out
 }
 
-invSize <- function(x, datatype="abundance", size=NULL){
+
+invSize <- function(x, datatype="abundance", size=NULL, conf=NULL){
   TYPE <- c("abundance", "incidence")
   if(is.na(pmatch(datatype, TYPE)))
     stop("invalid datatype")
@@ -149,44 +226,18 @@ invSize <- function(x, datatype="abundance", size=NULL){
       if(is.null(size)){
         size <- sum(x)
       }
-      method <- ifelse(size<sum(x), "rarefaction", ifelse(size==sum(x), "observation", "extrapolation"))
-      out <- data.frame(m=size, method=method, 
-                 SamCov=round(Chat.Ind(x,size),3),
-                 SpeRic=round(Dqhat.Ind(x,0,size),3),
-                 ShaDiv=round(Dqhat.Ind(x,1,size),3),
-                 SimDiv=round(Dqhat.Ind(x,2,size),3))
-	  colnames(out) <- c("m", "method", "SC", "q = 0", "q = 1", "q = 2")
-	  out
+      out <- invSize.Ind(x, size, conf)
     }
     else if(class(x)=="list"){
       if(is.null(size)){
         size <- min(unlist(lapply(x, function(x) sum(x))))
-      }    
-      
-      do.call(rbind, lapply(x, function(x){
-        method <- ifelse(size<sum(x), "rarefaction", ifelse(size==sum(x), "observation", "extrapolation"))
-        out <- data.frame(m=size, method=method, 
-                   SamCov=round(Chat.Ind(x,size),3),
-                   SpeRic=round(Dqhat.Ind(x,0,size),3),
-                   ShaDiv=round(Dqhat.Ind(x,1,size),3),
-                   SimDiv=round(Dqhat.Ind(x,2,size),3))
-		colnames(out) <- c("m", "method", "SC", "q = 0", "q = 1", "q = 2")
-		out
-      }))
+      }
+      out <- do.call(rbind, lapply(x, function(x) invSize.Ind(x, size, conf)))
     }else if(class(x)=="data.frame" | class(x)=="matrix"){
       if(is.null(size)){
         size <- min(unlist(apply(x, 2, function(x) sum(x))))
       }
-      do.call(rbind, apply(x, 2, function(x){
-        method <- ifelse(size<sum(x), "rarefaction", ifelse(size==sum(x), "observation", "extrapolation"))
-        out <- data.frame(m=size, method=method, 
-                   SamCov=round(Chat.Ind(x,size),3),
-                   SpeRic=round(Dqhat.Ind(x,0,size),3),
-                   ShaDiv=round(Dqhat.Ind(x,1,size),3),
-                   SimDiv=round(Dqhat.Ind(x,2,size),3))
-		colnames(out) <- c("m", "method", "SC", "q = 0", "q = 1", "q = 2")
-		out
-      }))
+      out <- do.call(rbind, apply(x, 2, function(x) invSize.Ind(x, size, conf)))
     }
   }else if(datatype=="incidence"){
     
@@ -194,46 +245,21 @@ invSize <- function(x, datatype="abundance", size=NULL){
       if(is.null(size)){
         size <- max(x)
       }
-      method <- ifelse(size<max(x), "rarefaction", ifelse(size==max(x), "observation", "extrapolation"))
-      out <- data.frame(t=size, method=method, 
-                 SamCov=round(Chat.Sam(x,size),3),
-                 SpeRic=round(Dqhat.Sam(x,0,size),3),
-                 ShaDiv=round(Dqhat.Sam(x,1,size),3),
-                 SimDiv=round(Dqhat.Sam(x,2,size),3))
-	  colnames(out) <- c("m", "method", "SC", "q = 0", "q = 1", "q = 2")
-	  out
+      out <- invSize.Sam(x, size, conf)
     }
     else if(class(x)=="list"){
       if(is.null(size)){
         size <- min(unlist(lapply(x, function(x) max(x))))
-      }
-      do.call(rbind, lapply(x, function(x){
-        method <- ifelse(size<max(x), "rarefaction", ifelse(size==max(x), "observation", "extrapolation"))
-        out <- data.frame(t=size, method=method, 
-                   SamCov=round(Chat.Sam(x,size),3),
-                   SpeRic=round(Dqhat.Sam(x,0,size),3),
-                   ShaDiv=round(Dqhat.Sam(x,1,size),3),
-                   SimDiv=round(Dqhat.Sam(x,2,size),3))
-		colnames(out) <- c("m", "method", "SC", "q = 0", "q = 1", "q = 2")
-		out
-      }))
+        }
+      out <- do.call(rbind, lapply(x, function(x) invSize.Sam(x, size, conf)))
     }else if(class(x)=="data.frame" | class(x)=="matrix"){
       if(is.null(size)){
         size <- min(unlist(apply(x, 2, function(x) max(x))))
       }
-      do.call(rbind, apply(x, 2, function(x){
-        method <- ifelse(size<max(x), "rarefaction", ifelse(size==max(x), "observation", "extrapolation"))
-        out <- data.frame(t=size, method=method, 
-                   SamCov=round(Chat.Sam(x,size),3),
-                   SpeRic=round(Dqhat.Sam(x,0,size),3),
-                   ShaDiv=round(Dqhat.Sam(x,1,size),3),
-                   SimDiv=round(Dqhat.Sam(x,2,size),3))
-		colnames(out) <- c("m", "method", "SC", "q = 0", "q = 1", "q = 2")
-		out
-      }))
+      out <- do.call(rbind, apply(x, 2, function(x) invSize.Sam(x, size, conf)))
     }
-    
   }
+  out
 }
 
 # 
@@ -256,20 +282,21 @@ invSize <- function(x, datatype="abundance", size=NULL){
 #' @param datatype data type of input data: individual-based abundance data (\code{datatype = "abundance"}),  
 #' sampling-unit-based incidence frequencies data (\code{datatype = "incidence_freq"}) or species by sampling-units incidence matrix (\code{datatype = "incidence_raw"}).
 #' @param base comparison base: sample-size-based (\code{base="size"}) or coverage-based \cr (\code{base="coverage"}).
-#' @param level an integer specifying a particular sample size or a number (between 0 and 1) specifying a particular value of sample coverage. 
+#' @param level an value specifying a particular sample size or a number (between 0 and 1) specifying a particular value of sample coverage. 
 #' If \code{base="size"} and \code{level=NULL}, then this function computes the diversity estimates for the minimum sample size among all sites. 
 #' If \code{base="coverage"} and \code{level=NULL}, then this function computes the diversity estimates for the minimum sample coverage among all sites. 
+#' @param conf a positive number < 1 specifying the level of confidence interval, default is 0.95. Remove C.I. by setting conf=NULL.
 #' @return a \code{data.frame} of species diversity table including the sample size, sample coverage,
 #' method (rarefaction or extrapolation), and diversity estimates with q = 0, 1, and 2 for the user-specified sample size or sample coverage.
 #' @examples
 #' data(spider)
-#' estimateD(spider, "abundance", base="size", level=NULL)
-#' estimateD(spider, "abundance", base="coverage", level=NULL)
+#' estimateD(spider, "abundance", base="size", level=NULL, conf=0.95)
+#' estimateD(spider, "abundance", base="coverage", level=NULL, conf=0.95)
 #' 
 #' data(ant)
-#' estimateD(ant, "incidence_freq", base="coverage", level=0.985)
+#' estimateD(ant, "incidence_freq", base="coverage", level=0.985, conf=NULL)
 #' @export
-estimateD <- function(x, datatype="abundance", base="size", level=NULL){
+estimateD <- function(x, datatype="abundance", base="size", level=NULL, conf=0.95){
   TYPE <- c("abundance", "incidence", "incidence_freq", "incidence_raw")
   #TYPE <- c("abundance", "incidence")
   if(is.na(pmatch(datatype, TYPE)))
@@ -284,6 +311,11 @@ estimateD <- function(x, datatype="abundance", base="size", level=NULL){
   }
   if(datatype=="incidence_freq") datatype <- "incidence"
   
+  if(datatype=="incidence_raw"){
+    if(class(x)=="data.frame" | class(x)=="matrix") x <- as.incfreq(x)
+    else if(class(x)=="list") x <- lapply(x, as.incfreq)
+    datatype <- "incidence"
+  }
     
   BASE <- c("size", "coverage")
   if(is.na(pmatch(base, BASE)))
@@ -293,22 +325,25 @@ estimateD <- function(x, datatype="abundance", base="size", level=NULL){
   base <- match.arg(base, BASE)
   
   if(base=="size"){
-    tmp <- invSize(x, datatype, size=level)
-    tmp <- cbind(Site=rownames(tmp), tmp)
-    rownames(tmp) <- NULL
-    tmp
+    tmp <- invSize(x, datatype, size=level, conf=conf)
   }else if(base=="coverage"){
-    tmp <- invChat(x, datatype, C=level)
-    tmp <- cbind(Site=rownames(tmp), tmp)
-    rownames(tmp) <- NULL
-    tmp
+    tmp <- invChat(x, datatype, C=level, conf=conf)
   }
+  
+  tmp <- tmp[!duplicated(tmp),]
+  
+  nam <- names(x)
+  if(is.null(nam)){
+    tmp
+  }else if(ncol(tmp)==6){
+    tmp <- cbind(site=nam, tmp)
+  }else{
+    tmp <- cbind(site=rep(nam, each=3), tmp)
+  }
+  rownames(tmp) <- NULL
+  tmp
 }
 
-# Compare(spider, datatype="abundance", base="size")
-# Compare(spider, datatype="abundance", base="coverage")
-# Compare(ant, datatype="incidence", base="size")
-# Compare(ant, datatype="incidence", base="coverage")
 
 
 # -----------------
